USE Performance;
GO

/*
Performance_AnchorVs3NF_Setup 2, 10, 100000;
-- 2 anchors, 10 attributes, 100000 rows takes about 5 minutes on my desktop
-- and generates 223MB data and 24MB logs 

Performance_AnchorVs3NF_Setup 2, 10, 200000;
-- 2 anchors, 10 attributes, 200000 rows takes about 16 minutes on my desktop
-- and generates 441MB data and 7MB logs 

Performance_AnchorVs3NF_Setup 2, 20, 100000;
-- 2 anchors, 20 attributes, 100000 rows takes about 13 minutes on my desktop
-- and generates 422MB data and 12MB logs 

Performance_AnchorVs3NF_Setup 2, 20, 200000;
-- 2 anchors, 20 attributes, 200000 rows takes about 30 minutes on my desktop
-- and generates 836MB data and 77MB logs 
*/

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Performance_AnchorVs3NF_Setup]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[Performance_AnchorVs3NF_Setup]
go

create procedure Performance_AnchorVs3NF_Setup
	/* initialization parameters */
	@anchors int,
	@attributes int,
	@rows int
as
begin

/* code generating anchors and attributes */
declare @sql varchar(max);

-- anchors
WITH anchors(num) AS (
	SELECT
		1
	UNION ALL
	SELECT
		num + 1
	FROM
		anchors
	WHERE
		num < @anchors
)
select 
	@sql = coalesce(@sql, '') + 
	'IF  EXISTS (
	SELECT 
		* 
	FROM 
		sys.objects 
	WHERE 
		object_id = OBJECT_ID(N''[dbo].' + anchorName + ''') 
	AND 
		type in (N''U'')
	)
	DROP TABLE [dbo].' + anchorName + '
	create table ' + anchorName + ' (
		' + anchorIdentityColumn + ' bigint not null,
		_metadata int not null
	);
	WITH row(num) AS (
	SELECT
		1
	UNION ALL
	SELECT
		num + 1
	FROM
		row
	WHERE
		num < ' + cast(@rows as varchar(10)) + '
	)
	insert into [dbo].' + anchorName + '
	select
		num, 
		1
	from
		row
	option (maxrecursion 0);
	ALTER TABLE [dbo].' + anchorName + ' ADD CONSTRAINT pk' + anchorMnemonic + ' PRIMARY KEY CLUSTERED (
			' + anchorIdentityColumn + ' asc
		); 
	UPDATE STATISTICS [dbo].' + anchorName + '
	'
from (
	select
		'[A' + cast(num as varchar(10)) + '_Anchor' + cast(num as varchar(10)) + ']' as anchorName,
		'A' + cast(num as varchar(10)) + '_ID' as anchorIdentityColumn,
		'A' + cast(num as varchar(10)) as anchorMnemonic
	from
		anchors
) t

exec(@sql);
set @sql = '';

-- attributes
WITH anchors(num) AS (
	SELECT
		1
	UNION ALL
	SELECT
		num + 1
	FROM
		anchors
	WHERE
		num < @anchors
),
attributes(num) AS (
	SELECT
		1
	UNION ALL
	SELECT
		num + 1
	FROM
		attributes
	WHERE
		num < @attributes
)
select 
	@sql = coalesce(@sql, '') + 	
	'IF  EXISTS (
	SELECT 
		* 
	FROM 
		sys.objects 
	WHERE 
		object_id = OBJECT_ID(N''[dbo].' + attributeName + ''') 
	AND 
		type in (N''U'')
	)
	DROP TABLE [dbo].' + attributeName + '
	create table ' + attributeName + ' (
		' + anchorIdentityColumn + ' bigint not null,
		' + attributeValueColumn + ' char(10) not null,
		' + attributeMnemonic + '_FromDate smalldatetime not null,
		_metadata int not null
	);
	WITH row(num) AS (
	SELECT
		1
	UNION ALL
	SELECT
		num + 2
	FROM
		row
	WHERE
		num < ' + cast(@rows as varchar(10)) + ' * 2 - 1
	)
	insert into [dbo].' + attributeName + '
	select
		(num + 1) / 2,
		left(cast(abs(checksum(newid())) as char(10)), 2),
		dateadd(day, ' + attributeNumber + ' % 2, ''2008-01-01''),
		1 + (' + attributeNumber + ' % 2)
	from
		row
	option (maxrecursion 0);
	WITH row(num) AS (
	SELECT
		2
	UNION ALL
	SELECT
		num + 2
	FROM
		row
	WHERE
		num < ' + cast(@rows as varchar(10)) + ' * 2
	)
	insert into [dbo].' + attributeName + '
	select
		num / 2, 
		left(cast(abs(checksum(newid())) as char(10)), 2),
		dateadd(day, ' + attributeNumber + ' % 2, ''2008-01-02''),
		2 + (' + attributeNumber + ' % 2)
	from
		row
	option (maxrecursion 0);
	ALTER TABLE [dbo].' + attributeName + ' ADD CONSTRAINT pk' + attributeMnemonic + ' PRIMARY KEY CLUSTERED (
			' + anchorIdentityColumn + ' asc,
			' + attributeMnemonic + '_FromDate desc
		);
	UPDATE STATISTICS [dbo].' + attributeName + '
	'
from (
	select
		cast(anchors.num as varchar(10)) as anchorNumber,
		cast(attributes.num as varchar(10)) as attributeNumber,
		'[A' + cast(anchors.num as varchar(10)) + '_Anchor' + cast(anchors.num as varchar(10)) + ']' as anchorName,
		'A' + cast(anchors.num as varchar(10)) + '_ID' as anchorIdentityColumn,
		'[A' + cast(anchors.num as varchar(10)) + 
		'a' + right('0' + cast(attributes.num as varchar(10)), 2) + 
		'_Anchor' + cast(anchors.num as varchar(10)) + 
		'Attribute' + right('0' + cast(attributes.num as varchar(10)), 2) + ']' as attributeName,
		'A' + cast(anchors.num as varchar(10)) + 
		'a' + right('0' + cast(attributes.num as varchar(10)), 2) + '_Value' as attributeValueColumn,
		'A' + cast(anchors.num as varchar(10)) + 
		'a' + right('0' + cast(attributes.num as varchar(10)), 2) as attributeMnemonic
	from
		anchors 
	cross join
		attributes
) t

exec(@sql);
set @sql = '';

/* tests
select top 10 * from dbo.A1a01_Anchor1Attribute01 order by A1_ID asc
select top 10 * from dbo.A1a02_Anchor1Attribute02 order by A1_ID asc
*/

-- ties
WITH anchors(num) AS (
	SELECT
		1
	UNION ALL
	SELECT
		num + 1
	FROM
		anchors
	WHERE
		num < @anchors
)
select
	@sql = coalesce(@sql, '') + 	
	'IF  EXISTS (
	SELECT 
		* 
	FROM 
		sys.objects 
	WHERE 
		object_id = OBJECT_ID(N''[dbo].' + tieName + ''') 
	AND 
		type in (N''U'')
	)
	DROP TABLE [dbo].' + tieName + '
	create table ' + tieName + ' (
		' + anchorIdentityColumn1 + ' bigint not null,
		' + anchorIdentityColumn2 + ' bigint not null,
		_metadata int not null
	);
	WITH row(num) AS (
	SELECT
		1
	UNION ALL
	SELECT
		num + 1
	FROM
		row
	WHERE
		num < ' + cast(@rows as varchar(10)) + '
	)
	insert into [dbo].' + tieName + '
	select
		num,
		' + cast(@rows as varchar(10)) + ' + 1 - num,
		1
	from
		row
	option (maxrecursion 0);
	ALTER TABLE [dbo].' + tieName + ' ADD CONSTRAINT pk' + anchorMnemonic1 + anchorMnemonic2 + ' PRIMARY KEY CLUSTERED (
			' + anchorIdentityColumn1 + ' asc,
			' + anchorIdentityColumn2 + ' asc
		);
	UPDATE STATISTICS [dbo].' + tieName + '
	'
from (
	select
		'A' + cast(a1.num as varchar(10)) + '_ID' as anchorIdentityColumn1,
		'A' + cast(a1.num as varchar(10)) as anchorMnemonic1,
		'A' + cast(a2.num as varchar(10)) + '_ID' as anchorIdentityColumn2,
		'A' + cast(a2.num as varchar(10)) as anchorMnemonic2, 
		'[A' + cast(a1.num as varchar(10)) + 'A' + cast(a2.num as varchar(10)) + 
		'_Anchor' + cast(a1.num as varchar(10)) + '_Anchor' + cast(a2.num as varchar(10)) + ']' as tieName
	from
		anchors a1
	cross join
		anchors a2
	where 
		a1.num <> a2.num
) t

exec(@sql);
set @sql = '';

/* tests
select top 10 * from dbo.A1A2_Anchor1Anchor2 order by A1_ID asc
select top 10 * from dbo.A1A2_Anchor1Anchor2 order by A1_ID desc
*/

-- latest views
WITH anchors(num) AS (
	SELECT
		1
	UNION ALL
	SELECT
		num + 1
	FROM
		anchors
	WHERE
		num < @anchors
),
attributes(num) AS (
	SELECT
		1
	UNION ALL
	SELECT
		num + 1
	FROM
		attributes
	WHERE
		num < @attributes
)
select 
	anchorNumber, 	
	'left join
		' + attributeName + ' ' + attributeMnemonic + '
	on
		' + attributeMnemonic + '.' + anchorIdentityColumn + ' = ' + anchorMnemonic + '.' + anchorIdentityColumn + '
	and
		' + attributeMnemonic + '.' + attributeMnemonic + '_FromDate = (
			select
				max(sub.' + attributeMnemonic + '_FromDate) 
			from
				' + attributeName + ' sub
			where
				sub.' + anchorIdentityColumn + ' = ' + anchorMnemonic + '.' + anchorIdentityColumn + '
		)
	' as joinClause
into
	#joins_latest
from (
	select
		anchors.num as anchorNumber,
		'[A' + cast(anchors.num as varchar(10)) + '_Anchor' + cast(anchors.num as varchar(10)) + ']' as anchorName,
		'A' + cast(anchors.num as varchar(10)) + '_ID' as anchorIdentityColumn,
		'A' + cast(anchors.num as varchar(10)) as anchorMnemonic,
		'[A' + cast(anchors.num as varchar(10)) + 
		'a' + right('0' + cast(attributes.num as varchar(10)), 2) + 
		'_Anchor' + cast(anchors.num as varchar(10)) + 
		'Attribute' + right('0' + cast(attributes.num as varchar(10)), 2) + ']' as attributeName,
		'A' + cast(anchors.num as varchar(10)) + 
		'a' + right('0' + cast(attributes.num as varchar(10)), 2) + '_Value' as attributeValueColumn,
		'A' + cast(anchors.num as varchar(10)) + 
		'a' + right('0' + cast(attributes.num as varchar(10)), 2) as attributeMnemonic
	from
		anchors 
	cross join
		attributes
) t;

/* tests
select * from #joins_latest
*/

WITH anchors(num) AS (
	SELECT
		1
	UNION ALL
	SELECT
		num + 1
	FROM
		anchors
	WHERE
		num < @anchors
),
attributes(num) AS (
	SELECT
		1
	UNION ALL
	SELECT
		num + 1
	FROM
		attributes
	WHERE
		num < @attributes
)
select 
	anchorNumber, 	
	attributeMnemonic + '.' + attributeValueColumn as columnReference
into
	#columns_latest
from (
	select
		anchors.num as anchorNumber,
		'[A' + cast(anchors.num as varchar(10)) + '_Anchor' + cast(anchors.num as varchar(10)) + ']' as anchorName,
		'A' + cast(anchors.num as varchar(10)) + '_ID' as anchorIdentityColumn,
		'A' + cast(anchors.num as varchar(10)) as anchorMnemonic,
		'[A' + cast(anchors.num as varchar(10)) + 
		'a' + right('0' + cast(attributes.num as varchar(10)), 2) + 
		'_Anchor' + cast(anchors.num as varchar(10)) + 
		'Attribute' + right('0' + cast(attributes.num as varchar(10)), 2) + ']' as attributeName,
		'A' + cast(anchors.num as varchar(10)) + 
		'a' + right('0' + cast(attributes.num as varchar(10)), 2) + '_Value' as attributeValueColumn,
		'A' + cast(anchors.num as varchar(10)) + 
		'a' + right('0' + cast(attributes.num as varchar(10)), 2) as attributeMnemonic
	from
		anchors 
	cross join
		attributes
) t;

/* tests
select * from #columns_latest
*/

declare @currentAnchor int;
set @currentAnchor = 1;

while @currentAnchor <= @anchors
begin
	declare @columns_latest varchar(max);
	set @columns_latest = '';

	select
		@columns_latest = @columns_latest + ',' + c.columnReference
	from
		#columns_latest c
	where
		c.anchorNumber = @currentAnchor

	declare @joins_latest varchar(max);
	set @joins_latest = '';

	select
		@joins_latest = @joins_latest + ' ' + j.joinClause
	from
		#joins_latest j
	where
		j.anchorNumber = @currentAnchor

	select
		@sql = coalesce(@sql, '') + 	
		'
		IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N''[dbo].' + viewName + '''))
		DROP VIEW [dbo].' + viewName + '
		;'
	from (
		select
			'[A' + cast(@currentAnchor as varchar(10)) + '_Anchor' + cast(@currentAnchor as varchar(10)) + ']' as anchorName,
			'A' + cast(@currentAnchor as varchar(10)) + '_ID' as anchorIdentityColumn,
			'A' + cast(@currentAnchor as varchar(10)) as anchorMnemonic,
			'[lA' + cast(@currentAnchor as varchar(10)) + '_Anchor' + cast(@currentAnchor as varchar(10)) + ']' as viewName
	) t

	exec(@sql);
	set @sql = '';

	select
		@sql = coalesce(@sql, '') + 	
		'
		create view ' + viewName + ' as
		select
			' + anchorMnemonic + '.' + anchorIdentityColumn + '
			' + @columns_latest + '
		from
			' + anchorName + ' ' +  anchorMnemonic + '
		' + @joins_latest + '
		;'
	from (
		select
			'[A' + cast(@currentAnchor as varchar(10)) + '_Anchor' + cast(@currentAnchor as varchar(10)) + ']' as anchorName,
			'A' + cast(@currentAnchor as varchar(10)) + '_ID' as anchorIdentityColumn,
			'A' + cast(@currentAnchor as varchar(10)) as anchorMnemonic,
			'[lA' + cast(@currentAnchor as varchar(10)) + '_Anchor' + cast(@currentAnchor as varchar(10)) + ']' as viewName
	) t

	exec(@sql);
	set @sql = '';

	set @currentAnchor = @currentAnchor + 1
end;

/* tests
select * from lA1_Anchor1
*/

-- temporal functions
WITH anchors(num) AS (
	SELECT
		1
	UNION ALL
	SELECT
		num + 1
	FROM
		anchors
	WHERE
		num < @anchors
),
attributes(num) AS (
	SELECT
		1
	UNION ALL
	SELECT
		num + 1
	FROM
		attributes
	WHERE
		num < @attributes
)
select 
	anchorNumber, 	
	'left join
		' + attributeName + ' ' + attributeMnemonic + '
	on
		' + attributeMnemonic + '.' + anchorIdentityColumn + ' = ' + anchorMnemonic + '.' + anchorIdentityColumn + '
	and
		' + attributeMnemonic + '.' + attributeMnemonic + '_FromDate = (
			select
				max(sub.' + attributeMnemonic + '_FromDate) 
			from
				' + attributeName + ' sub
			where
				sub.' + anchorIdentityColumn + ' = ' + anchorMnemonic + '.' + anchorIdentityColumn + '
			and
				sub.' + attributeMnemonic + '_FromDate <= @requestedDate
		)
	' as joinClause
into
	#joins_temporal
from (
	select
		anchors.num as anchorNumber,
		'[A' + cast(anchors.num as varchar(10)) + '_Anchor' + cast(anchors.num as varchar(10)) + ']' as anchorName,
		'A' + cast(anchors.num as varchar(10)) + '_ID' as anchorIdentityColumn,
		'A' + cast(anchors.num as varchar(10)) as anchorMnemonic,
		'[A' + cast(anchors.num as varchar(10)) + 
		'a' + right('0' + cast(attributes.num as varchar(10)), 2) + 
		'_Anchor' + cast(anchors.num as varchar(10)) + 
		'Attribute' + right('0' + cast(attributes.num as varchar(10)), 2) + ']' as attributeName,
		'A' + cast(anchors.num as varchar(10)) + 
		'a' + right('0' + cast(attributes.num as varchar(10)), 2) + '_Value' as attributeValueColumn,
		'A' + cast(anchors.num as varchar(10)) + 
		'a' + right('0' + cast(attributes.num as varchar(10)), 2) as attributeMnemonic
	from
		anchors 
	cross join
		attributes
) t;

/* tests
select * from #joins_temporal
*/

WITH anchors(num) AS (
	SELECT
		1
	UNION ALL
	SELECT
		num + 1
	FROM
		anchors
	WHERE
		num < @anchors
),
attributes(num) AS (
	SELECT
		1
	UNION ALL
	SELECT
		num + 1
	FROM
		attributes
	WHERE
		num < @attributes
)
select 
	anchorNumber, 	
	attributeMnemonic + '.' + attributeValueColumn as columnReference
into
	#columns_temporal
from (
	select
		anchors.num as anchorNumber,
		'[A' + cast(anchors.num as varchar(10)) + '_Anchor' + cast(anchors.num as varchar(10)) + ']' as anchorName,
		'A' + cast(anchors.num as varchar(10)) + '_ID' as anchorIdentityColumn,
		'A' + cast(anchors.num as varchar(10)) as anchorMnemonic,
		'[A' + cast(anchors.num as varchar(10)) + 
		'a' + right('0' + cast(attributes.num as varchar(10)), 2) + 
		'_Anchor' + cast(anchors.num as varchar(10)) + 
		'Attribute' + right('0' + cast(attributes.num as varchar(10)), 2) + ']' as attributeName,
		'A' + cast(anchors.num as varchar(10)) + 
		'a' + right('0' + cast(attributes.num as varchar(10)), 2) + '_Value' as attributeValueColumn,
		'A' + cast(anchors.num as varchar(10)) + 
		'a' + right('0' + cast(attributes.num as varchar(10)), 2) as attributeMnemonic
	from
		anchors 
	cross join
		attributes
) t;

/* tests
select * from #columns_temporal
*/

WITH anchors(num) AS (
	SELECT
		1
	UNION ALL
	SELECT
		num + 1
	FROM
		anchors
	WHERE
		num < @anchors
),
attributes(num) AS (
	SELECT
		1
	UNION ALL
	SELECT
		num + 1
	FROM
		attributes
	WHERE
		num < @attributes
)
select 
	anchorNumber, 	
	attributeMnemonic + '.' + attributeValueColumn + ' is not null' as whereCondition
into
	#conditions_temporal
from (
	select
		anchors.num as anchorNumber,
		'[A' + cast(anchors.num as varchar(10)) + '_Anchor' + cast(anchors.num as varchar(10)) + ']' as anchorName,
		'A' + cast(anchors.num as varchar(10)) + '_ID' as anchorIdentityColumn,
		'A' + cast(anchors.num as varchar(10)) as anchorMnemonic,
		'[A' + cast(anchors.num as varchar(10)) + 
		'a' + right('0' + cast(attributes.num as varchar(10)), 2) + 
		'_Anchor' + cast(anchors.num as varchar(10)) + 
		'Attribute' + right('0' + cast(attributes.num as varchar(10)), 2) + ']' as attributeName,
		'A' + cast(anchors.num as varchar(10)) + 
		'a' + right('0' + cast(attributes.num as varchar(10)), 2) + '_Value' as attributeValueColumn,
		'A' + cast(anchors.num as varchar(10)) + 
		'a' + right('0' + cast(attributes.num as varchar(10)), 2) as attributeMnemonic
	from
		anchors 
	cross join
		attributes
) t;

set @currentAnchor = 1;

while @currentAnchor <= @anchors
begin
	declare @columns_temporal varchar(max);
	set @columns_temporal = '';

	select
		@columns_temporal = @columns_temporal + ',' + c.columnReference
	from
		#columns_temporal c
	where
		c.anchorNumber = @currentAnchor

	declare @joins_temporal varchar(max);
	set @joins_temporal = '';

	select
		@joins_temporal = @joins_temporal + ' ' + j.joinClause
	from
		#joins_temporal j
	where
		j.anchorNumber = @currentAnchor

	declare @conditions_temporal varchar(max);
	set @conditions_temporal = '';

	select
		@conditions_temporal = @conditions_temporal + case when @conditions_temporal = '' then '' else ' or ' end + c.whereCondition
	from
		#conditions_temporal c
	where
		c.anchorNumber = @currentAnchor

	select
		@sql = coalesce(@sql, '') + 	
		'
		IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N''[dbo].' + functionName + ''') AND type in (N''FN'', N''IF'', N''TF'', N''FS'', N''FT''))
		DROP FUNCTION [dbo].' + functionName + '
		;'
	from (
		select
			'[A' + cast(@currentAnchor as varchar(10)) + '_Anchor' + cast(@currentAnchor as varchar(10)) + ']' as anchorName,
			'A' + cast(@currentAnchor as varchar(10)) + '_ID' as anchorIdentityColumn,
			'A' + cast(@currentAnchor as varchar(10)) as anchorMnemonic,
			'[fA' + cast(@currentAnchor as varchar(10)) + '_Anchor' + cast(@currentAnchor as varchar(10)) + ']' as functionName
	) t

	exec(@sql);
	set @sql = '';

	select
		@sql = coalesce(@sql, '') + 	
		'
		create function ' + functionName + ' (
			@requestedDate datetime
		)
		returns table 
		return
		select
			' + anchorMnemonic + '.' + anchorIdentityColumn + '
			' + @columns_temporal + '
		from
			' + anchorName + ' ' +  anchorMnemonic + '
		' + @joins_temporal + '
		where
			' + @conditions_temporal + '
		;'
	from (
		select
			'[A' + cast(@currentAnchor as varchar(10)) + '_Anchor' + cast(@currentAnchor as varchar(10)) + ']' as anchorName,
			'A' + cast(@currentAnchor as varchar(10)) + '_ID' as anchorIdentityColumn,
			'A' + cast(@currentAnchor as varchar(10)) as anchorMnemonic,
			'[fA' + cast(@currentAnchor as varchar(10)) + '_Anchor' + cast(@currentAnchor as varchar(10)) + ']' as functionName
	) t

	exec(@sql);
	set @sql = '';

	set @currentAnchor = @currentAnchor + 1
end;

/* tests
select * from dbo.fA1_Anchor1('2008-01-01')
select * from dbo.fA1_Anchor1('2008-01-02')
select * from dbo.fA1_Anchor1('2008-01-03')
*/

WITH anchors(num) AS (
	SELECT
		1
	UNION ALL
	SELECT
		num + 1
	FROM
		anchors
	WHERE
		num < @anchors
),
attributes(num) AS (
	SELECT
		1
	UNION ALL
	SELECT
		num + 1
	FROM
		attributes
	WHERE
		num < @attributes
)
select 
	anchorNumber,
	'
	select distinct 
		' + attributeMnemonic + '_FromDate as fromDate
	from
		' + attributeName + '
	' as historizationQuery
into
	#historization_queries
from (
	select
		anchors.num as anchorNumber,
		'[A' + cast(anchors.num as varchar(10)) + '_Anchor' + cast(anchors.num as varchar(10)) + ']' as anchorName,
		'A' + cast(anchors.num as varchar(10)) + '_ID' as anchorIdentityColumn,
		'A' + cast(anchors.num as varchar(10)) as anchorMnemonic,
		'[A' + cast(anchors.num as varchar(10)) + 
		'a' + right('0' + cast(attributes.num as varchar(10)), 2) + 
		'_Anchor' + cast(anchors.num as varchar(10)) + 
		'Attribute' + right('0' + cast(attributes.num as varchar(10)), 2) + ']' as attributeName,
		'A' + cast(anchors.num as varchar(10)) + 
		'a' + right('0' + cast(attributes.num as varchar(10)), 2) + '_Value' as attributeValueColumn,
		'A' + cast(anchors.num as varchar(10)) + 
		'a' + right('0' + cast(attributes.num as varchar(10)), 2) as attributeMnemonic
	from
		anchors 
	cross join
		attributes
) t;

/* tests
select * from #historization_queries
*/

set @currentAnchor = 1;

while @currentAnchor <= @anchors
begin
	declare @historizationQuery varchar(max);
	set @historizationQuery = '';

	select
		@historizationQuery = @historizationQuery + case when @historizationQuery = '' then '' else ' union ' end + h.historizationQuery
	from
		#historization_queries h
	where
		h.anchorNumber = @currentAnchor
		
	set @sql = 
	'IF EXISTS (SELECT *
          FROM tempdb..sysobjects
          WHERE type = ''U'' and NAME = ''##usedDates'') drop table ##usedDates;' +
	'select * into ##usedDates from (' + @historizationQuery + ') t;'

	exec(@sql);
	set @sql = '';
	
	declare @3nfName varchar(max);
	select @3nfName = 'A' + cast(@currentAnchor as varchar(10)) + '_Anchor' + cast(@currentAnchor as varchar(10)) + '_3NF';
	
	declare @functionName varchar(max);
	select @functionName = 'fA' + cast(@currentAnchor as varchar(10)) + '_Anchor' + cast(@currentAnchor as varchar(10))
		
	select @sql = 
	'IF  EXISTS (
	SELECT 
		* 
	FROM 
		sys.objects 
	WHERE 
		object_id = OBJECT_ID(N''[dbo].' + @3nfName + ''') 
	AND 
		type in (N''U'')
	)
	DROP TABLE [dbo].' + @3nfName + ';' + 
	'select top 0 *, isnull(cast(null as datetime), ''1901-01-01'') as fromDate into ' + @3nfName + ' from ' + @functionName + '(null);'

	exec(@sql);
	set @sql = '';
		
	declare @usedDate datetime;
	declare dateCursor cursor for select * from ##usedDates;
	open dateCursor;
	
	fetch next from dateCursor into @usedDate;
	while @@FETCH_STATUS = 0
	begin
		select @sql = 'insert into ' + @3nfName +
		' select *, ''' + convert(char(10), @usedDate, 121) + 
		''' from ' + @functionName + '(''' + convert(char(10), @usedDate, 121) + ''');'

		exec(@sql);
		set @sql = '';

		fetch next from dateCursor into @usedDate;	
	end
	close dateCursor;
	deallocate dateCursor;
		
	set @sql ='
	ALTER TABLE ' + @3nfName + ' ADD CONSTRAINT pkA' + cast(@currentAnchor as varchar(10)) + '_3NF PRIMARY KEY CLUSTERED (' +
		'A' + cast(@currentAnchor as varchar(10)) + '_ID, fromDate
		); 
	UPDATE STATISTICS ' + @3nfName + '
	'

	exec(@sql);

	set @currentAnchor = @currentAnchor + 1;
end

/* tests
select * from dbo.A1_Anchor1_3NF
*/
end
