USE Performance;
GO

/*
Performance_AnchorVs3NF_Test 10
*/

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[lA1_Anchor1]'))
DROP VIEW [dbo].[lA1_Anchor1]
GO

create view lA1_Anchor1 as 
select 
	1 as A1_ID,
	'X' as A1a01_Value,
	'X' as A1a02_Value;
go

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[A1A2_Anchor1_Anchor2]') AND type in (N'U'))
DROP TABLE [dbo].[A1A2_Anchor1_Anchor2]
GO

select 
	1 as A1_ID,
	1 as A2_ID,
	1 as _metadata
into
	A1A2_Anchor1_Anchor2
go

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[lA2_Anchor2]'))
DROP VIEW [dbo].[lA2_Anchor2]
GO

create view lA2_Anchor2 as 
select 
	1 as A2_ID,
	'X' as A2a01_Value,
	'X' as A2a02_Value;
go

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[A1_Anchor1_3NF]') AND type in (N'U'))
DROP TABLE [dbo].[A1_Anchor1_3NF]
GO

select
	1 as A1_ID,
	'X' as A1a01_Value,
	'X' as A1a02_Value,
	'2008-01-01' as fromDate
into
	A1_Anchor1_3NF;
go

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[A2_Anchor1_3NF]') AND type in (N'U'))
DROP TABLE [dbo].[A2_Anchor1_3NF]
GO

select
	1 as A2_ID,
	'X' as A2a01_Value,
	'X' as A2a02_Value,
	'2008-01-01' as fromDate
into
	A2_Anchor1_3NF;
go

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Performance_AnchorVs3NF_Test]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[Performance_AnchorVs3NF_Test]
go

create procedure Performance_AnchorVs3NF_Test
	@benchmarkRuns int
as
begin

declare @iter int;
set @iter = 0;

-- performance: anchor model
declare @result table (
	value1 char(10),
	value2 char(10),
	value3 char(10),
	value4 char(10),
	occurances int
);
declare @timers table (
	test varchar(100),
	startTime datetime,
	stopTime datetime
);

insert into @timers (test, startTime) values('Anchor model single attributes', GETDATE())
while @iter < @benchmarkRuns
begin
	DBCC FREEPROCCACHE;
	DBCC DROPCLEANBUFFERS;
	insert into @result (value1, occurances)
	select
		lA1.A1a01_Value, 
		count(distinct lA2.A2a01_Value) 
	from -- select top 100 * from
		dbo.lA1_Anchor1 lA1
	join -- select top 100 * from 
		dbo.A1A2_Anchor1_Anchor2 A1A2
	on
		A1A2.A1_ID = lA1.A1_ID
	join -- select top 100 * from
		dbo.lA2_Anchor2 lA2
	on
		lA2.A2_ID = A1A2.A2_ID
	group by
		lA1.A1a01_Value
	order by
		2 desc;
		
	set @iter = @iter + 1;
end
update @timers set stopTime = GETDATE() where test = 'Anchor model single attributes'

set @iter = 0;
delete from @result;

-- performance: 3NF
insert into @timers (test, startTime) values('3NF single attributes', GETDATE())
while @iter < @benchmarkRuns
begin
	DBCC FREEPROCCACHE;
	DBCC DROPCLEANBUFFERS;
	insert into @result (value1, occurances)
	select
		A1.A1a01_Value, 
		count(distinct A2.A2a01_Value) 
	from
		dbo.A1_Anchor1_3NF A1
	join
		dbo.A1A2_Anchor1_Anchor2 A1A2
	on
		A1A2.A1_ID = A1.A1_ID
	join
		dbo.A2_Anchor2_3NF A2
	on
		A2.A2_ID = A1A2.A2_ID
	where
		A1.fromDate = (
			select 
				MAX(fromDate)
			from
				dbo.A1_Anchor1_3NF sub
			where
				sub.A1_ID = A1.A1_ID
		)
	and
		A2.fromDate = (
			select 
				MAX(fromDate)
			from
				dbo.A2_Anchor2_3NF sub
			where
				sub.A2_ID = A2.A2_ID
		)	
	group by
		A1.A1a01_Value
	order by
		2 desc	

	set @iter = @iter + 1;
end
update @timers set stopTime = GETDATE() where test = '3NF single attributes'

set @iter = 0;
delete from @result;

-- performance: anchor model (2 groupings)
insert into @timers (test, startTime) values('Anchor model double attributes', GETDATE())
while @iter < @benchmarkRuns
begin
	DBCC FREEPROCCACHE;
	DBCC DROPCLEANBUFFERS;
	insert into @result (value1, value2, occurances)
	select
		lA1.A1a01_Value, 
		lA1.A1a02_Value, 
		count(distinct lA2.A2a01_Value) 
	from
		dbo.lA1_Anchor1 lA1
	join
		dbo.A1A2_Anchor1_Anchor2 A1A2
	on
		A1A2.A1_ID = lA1.A1_ID
	join
		dbo.lA2_Anchor2 lA2
	on
		lA2.A2_ID = A1A2.A2_ID
	group by
		lA1.A1a01_Value,
		lA1.A1a02_Value
	order by
		2 desc;
		
	set @iter = @iter + 1;
end
update @timers set stopTime = GETDATE() where test = 'Anchor model double attributes'

set @iter = 0;
delete from @result;

-- performance: 3NF
insert into @timers (test, startTime) values('3NF double attributes', GETDATE())
while @iter < @benchmarkRuns
begin
	DBCC FREEPROCCACHE;
	DBCC DROPCLEANBUFFERS;
	insert into @result (value1, value2, occurances)
	select
		A1.A1a01_Value, 
		A1.A1a02_Value, 
		count(distinct A2.A2a01_Value) 
	from
		dbo.A1_Anchor1_3NF A1
	join
		dbo.A1A2_Anchor1_Anchor2 A1A2
	on
		A1A2.A1_ID = A1.A1_ID
	join
		dbo.A2_Anchor2_3NF A2
	on
		A2.A2_ID = A1A2.A2_ID
	where
		A1.fromDate = (
			select 
				MAX(fromDate)
			from
				dbo.A1_Anchor1_3NF sub
			where
				sub.A1_ID = A1.A1_ID
		)
	and
		A2.fromDate = (
			select 
				MAX(fromDate)
			from
				dbo.A2_Anchor2_3NF sub
			where
				sub.A2_ID = A2.A2_ID
		)	
	group by
		A1.A1a01_Value,
		A1.A1a02_Value
	order by
		2 desc	

	set @iter = @iter + 1;
end
update @timers set stopTime = GETDATE() where test = '3NF double attributes'


select * from @timers;

end